use anyhow::{Context, Result};
use rumqttc::{Client, MqttOptions, QoS, Transport};
use serde::{Serialize, Deserialize};
use serde_json;
use std::io::{self, Read};
use std::time::Duration;
use std::thread;
use serialport;
use actix_web::{web, App, HttpServer, HttpResponse};
use actix_cors::Cors;
use std::fs;
use std::sync::{Arc, Mutex};

#[derive(Serialize, Deserialize)]
struct WeightData {
    weight: String,
    timestamp: u64,
}

#[derive(Serialize)]
struct MacResponse {
    mac_address: String,
}

#[derive(Serialize)]
struct ScaleTypeResponse {
    scale_type: String,
}

#[derive(Deserialize, Clone)]
struct MqttConfig {
    client_id: String,
    host: String,
    port: u16,
    username: String,
    password: String,
    keep_alive_secs: u64,
    topic_prefix: String,
}

#[derive(Deserialize)]
struct Config {
    scale_type: String,
    serial_port: String,
    baud_rate: u32,
    mqtt: MqttConfig,
}

fn read_config() -> Result<Config> {
    let config_path = "config.json";
    let content = fs::read_to_string(config_path)
        .with_context(|| format!("无法读取配置文件 {}", config_path))?;
    let config: Config = serde_json::from_str(&content)
        .with_context(|| format!("配置文件格式错误 {}", config_path))?;
    Ok(config)
}

fn get_mac_address() -> String {
    match read_config() {
        Ok(config) => config.scale_type,
        Err(_) => String::from("Unknown")
    }
}

fn get_scale_type() -> String {
    match read_config() {
        Ok(config) => config.scale_type,
        Err(_) => String::from("Unknown")
    }
}

// HTTP处理函数
async fn get_mac() -> HttpResponse {
    let mac = get_mac_address();
    println!("HTTP请求：获取MAC地址 = {}", mac);
    let response = MacResponse {
        mac_address: mac,
    };
     // 在新线程中运行MQTT和串口服
     let _mqtt_handle = tokio::spawn(async {
        if let Err(e) = run_mqtt_and_serial().await {
            eprintln!("MQTT/串口服务错误: {}", e);
        }
    });
    
    
    HttpResponse::Ok().json(response)
}

// 新增的HTTP处理函数
async fn get_scale() -> HttpResponse {
    let scale_type = get_scale_type();
    println!("HTTP请求：获取天平类= {}", scale_type);
    let response = ScaleTypeResponse {
        scale_type,
    };
    HttpResponse::Ok().json(response)
}

#[actix_web::main]
async fn main() -> Result<()> {
    println!("程序启动...");
    
    // 启动HTTP服务
    println!("正在启动HTTP服务..");
    
    // 创建HTTP服务
    let server = HttpServer::new(|| {
        // 配置CORS
        let cors = Cors::default()
            .allow_any_origin()  // 允许所有来
            .allow_any_method()  // 允许所有HTTP方法
            .allow_any_header()  // 允许所有请求头
            .max_age(3600);      // 设置预检请求的缓存时间（秒）
            
        App::new()
            .wrap(cors)  // 添加CORS中间
            .route("/mac", web::get().to(get_mac))
            .route("/scale", web::get().to(get_scale))  // 新增的路
    })
    .bind(("0.0.0.0", 8080))?
    .run();
    
    println!("HTTP服务器已启动，监听在 http://127.0.0.1:8080");
    
   
    // 等待HTTP服务器结
    server.await?;
    
    Ok(())
}

// MQTT和串口处理函
async fn run_mqtt_and_serial() -> Result<()> {
    // 读取配置
    let config = read_config()?;
    
    // 检查串口是否被占用
    println!("正在检查串{} 是否可用...", config.serial_port);
    match serialport::new(&config.serial_port, config.baud_rate)
        .timeout(Duration::from_millis(10))
        .open() {
        Err(_e) => {
            eprintln!("串口 {} 已被占用", config.serial_port);
            return Ok(());
        },
        Ok(_) => {}
    }
    
    let mqtt_config = config.mqtt.clone();
    
    // 创建MQTT连接函数
    let create_mqtt_client = move || -> (Client, rumqttc::Connection) {
        let mut mqttopts = MqttOptions::new(
            &mqtt_config.client_id,
            &mqtt_config.host,
            mqtt_config.port
        );
    mqttopts.set_keep_alive(Duration::from_secs(60)); // 增加keep-alive时间0
        mqttopts.set_credentials(&mqtt_config.username, &mqtt_config.password);

    // 初始化MQTT客户
    let (client, mut connection) = create_mqtt_client();
    let client = Arc::new(Mutex::new(client));
    let client_clone = client.clone();

    // 在单独的线程中处理MQTT连接
    thread::spawn(move || {
        loop {  // 添加无限循环来保持重
            for notification in connection.iter() {
                match notification {
                    Ok(_) => {},
                    Err(e) => {
                        eprintln!("MQTT连接错误: {:?}", e);
                        // 短暂等待后继续尝试重
                        thread::sleep(Duration::from_secs(5));
                        let (new_client, new_connection) = create_mqtt_client();
                        *client_clone.lock().unwrap() = new_client;
                        connection = new_connection;
                        break;
                    }
                }
            }
            eprintln!("MQTT连接断开秒后尝试重连...");
            thread::sleep(Duration::from_secs(5));
            }
        }
    });

    // 打开配置的串
    println!("正在尝试打开串口 {}...", config.serial_port);
    let mut port = serialport::new(&config.serial_port, config.baud_rate)
        .timeout(Duration::from_millis(1000))
        .open()
        .with_context(|| format!("无法打开串口 {}", config.serial_port))?;

    println!("成功打开串口 {}!", config.serial_port);
    println!("正在读取数据... (Ctrl+C 退");

    // 增加缓冲区大小并使用String来累积数
    let mut serial_buf: Vec<u8> = vec![0; 1024];
    let mut accumulated_data = String::new();
    
    loop {
        match port.read(serial_buf.as_mut_slice()) {
            Ok(t) => {
                if t > 0 {
                    if let Ok(data) = String::from_utf8(serial_buf[..t].to_vec()) {
                        accumulated_data.push_str(&data);
                        
                        if accumulated_data.contains('\n') || accumulated_data.contains('\r') {
                            let lines: Vec<&str> = accumulated_data
                                .split(|c| c == '\n' || c == '\r')
                                .filter(|s| !s.is_empty())
                                .collect();
                            
                            for line in lines {
                                let trimmed_data = line.trim();
                                if !trimmed_data.is_empty() {
                                    println!("收到完整数据: {}", trimmed_data);
                                    
                                    let weight_data = WeightData {
                                        weight: trimmed_data.to_string(),
                                        timestamp: std::time::SystemTime::now()
                                            .duration_since(std::time::UNIX_EPOCH)
                                            .unwrap()
                                            .as_secs(),
                                    };

                                    let json_data = serde_json::to_string(&weight_data)?;
                                    println!("当前JSON数据: {}", json_data);
                                    
                                    let mac_address = get_mac_address();
                                    println!("MAC地址: {}", mac_address);
                                    
                                    let topic = format!("{}/{}", mqtt_config.topic_prefix, mac_address);
                                    let mut retry_count = 0;
                                    let max_retries = 3;
                                    
                                    // 添加重试机制
                                    while retry_count < max_retries {
                                        let mut client_guard = client.lock().unwrap();
                                        match client_guard.publish(&topic, QoS::AtLeastOnce, false, json_data.clone()) {
                                            Ok(_) => {
                                                println!("成功发送数据到MQTT");
                                                break;
                                            }
                                            Err(e) => {
                                                eprintln!("MQTT发布错误 (尝试 {}/{}): {:?}", retry_count + 1, max_retries, e);
                                                retry_count += 1;
                                                if retry_count < max_retries {
                                                    thread::sleep(Duration::from_secs(1));
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            
                            accumulated_data.clear();
                        }
                    }
                }
            }
            Err(ref e) if e.kind() == io::ErrorKind::TimedOut => {
                continue;
            }
            Err(e) => {
                eprintln!("发生错误: {}", e);
                break;
            }
        }
    }

    Ok(())
}
